@@ -137,8 +137,12 @@ config = {
 			"item_key": { "type": "keyword"},
 			"score": { "type": "float"},
 			"tmf": { "type": "float"},
+			"url": { "type": "keyword", "index": "false"},
+			"tms": { "type": "keyword", "index": "false"},
 			"stroke": { "type": "keyword", "index": "false"},
 			"strokes": { "type": "keyword", "index": "false"},
+			"text": { "type": "keyword", "index": "false"},
+			"labels": { "type": "keyword"}, # for annotation 
 			"_snt": { "type": "keyword", "index": "false"},
 			"_ske": { "type": "keyword", "index": "false"},
 			"en": { "type": "text", "analyzer": "standard","fielddata":"true"},
@@ -205,11 +209,13 @@ config = {
 			"fd": { "type": "keyword", "index": "false"},
 			"short_msg": { "type": "keyword", "index": "false"},
 			"arr": { "type": "keyword", "index": "false" }, # dim arr of dsk
-			"info": { "type": "keyword", "index": "false" ,"store": "false"},
-			"kw": { "type": "keyword", "index": "false" ,"store": "false", "ignore_above": 60},
-			"meta": { "type": "keyword", "index": "false" ,"store": "false"},
-			"dim": { "type": "keyword", "index": "false" },
-			"dims": { "type": "keyword", "index": "false" },
+			"info": { "type": "keyword", "index": "false" ,"ignore_above": 50},
+			"kw": { "type": "keyword", "index": "false" , "ignore_above": 50},
+			"meta": { "type": "keyword", "index": "false", "ignore_above": 50},
+			"dim": { "type": "keyword", "index": "false" , "ignore_above": 50},
+			"dims": { "type": "keyword", "index": "false" , "ignore_above": 50},
+			"meta": { "type": "keyword", "index": "false", "ignore_above": 50},
+			"mkf": { "type": "keyword", "index": "false", "ignore_above": 50 },
 			"tc": {"type": "integer" , "index": "false"},
 			"sc": {"type": "integer" , "index": "false"},
 			"isum": {"type": "integer" , "index": "false"},
@@ -228,6 +234,7 @@ config = {
 			"tsv": { "type": "keyword",  "index": "false"},
 			"pair": { "type": "keyword",  "index": "false"},
 			"json": { "type": "keyword",  "index": "false"},
+			"dsk": { "type": "keyword",  "index": "false"},
 			"v": { "type": "keyword",  "index": "false"},
 			"n": { "type": "keyword",  "index": "false"},
 			"adj": { "type": "keyword",  "index": "false"},
