@@ -38,6 +38,11 @@ async def input_item(request: Request):
 async def getdata(fname:str="first name", lname:str="last name"):
 	return { "fname":fname, 'lname':lname }
 
+@app.get("/svnup")
+async def svnup():
+	''' 2022.8.6 '''
+	return os.system(f"svn up /data/cikuu --username zhangyue --password zhangy1235")
+
 def run(port, reload:bool=False): 
 	''' python3 __main__.py 8000 --reload true '''
 	uvicorn.run("__main__:app", host='0.0.0.0', port=port, reload=reload) 	#uvicorn.run(app, host='0.0.0.0', port=port, reload=reload)
